#!/usr/bin/env python

"""
Import utilities.

Copyright (C) 2017, 2018 Paul Boddie <paul@boddie.org.uk>

This program is free software; you can redistribute it and/or modify it under
the terms of the GNU General Public License as published by the Free Software
Foundation; either version 3 of the License, or (at your option) any later
version.

This program is distributed in the hope that it will be useful, but WITHOUT
ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
details.

You should have received a copy of the GNU General Public License along with
this program.  If not, see <http://www.gnu.org/licenses/>.
"""

from os.path import isdir, join, split, splitext
from os import listdir
from importlib import import_module

reserved = ["__init__", "common", "manifest"]

def get_extensions(dirname, modname, stores, reserved, prefix=None):

    "Import extensions inside 'dirname'."

    for filename in listdir(dirname):
        pathname = join(dirname, filename)

        # Descend into directories, prefixing the identified modules.

        if isdir(pathname):
            store_name = prefix and "%s.%s" % (prefix, modname) or filename
            get_extensions(pathname, "%s.%s" % (modname, filename),
                           stores, reserved, store_name)
            continue

        # Identify modules and import them.

        leafname, ext = splitext(filename)

        if ext == ".py" and leafname not in reserved:
            store_name = prefix and "%s.%s" % (prefix, leafname) or leafname
            stores[store_name] = import_module("%s.%s" % (modname, leafname))

def get_mapping(modules, get_key, get_value):

    """
    Using the 'modules' mapping, employ 'get_key' and 'get_value' to register
    objects provided by the modules in a mapping.
    """

    mapping = {}

    # Use the callables to obtain the keys and values from modules.

    for name, module in modules.items():
        mapping[get_key(name, module)] = get_value(module)

    return mapping

def get_modules(module_file, module_name):

    """
    Obtain details of the indicated module's package using 'module_file',
    corresponding to a __file__ attribute, and 'module_name', corresponding to
    a __name__ attribute.
    """

    dirname = split(module_file)[0]
    package = module_name.rsplit(".", 1)[0]

    # Define an attribute mapping names to modules.

    modules = {}
    get_extensions(dirname, package, modules, reserved)
    return modules

# vim: tabstop=4 expandtab shiftwidth=4
