#!/usr/bin/env python

"""
Stand-alone input context.

Copyright (C) 2018, 2019 Paul Boddie <paul@boddie.org.uk>

This program is free software; you can redistribute it and/or modify it under
the terms of the GNU General Public License as published by the Free Software
Foundation; either version 3 of the License, or (at your option) any later
version.

This program is distributed in the hope that it will be useful, but WITHOUT
ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
details.

You should have received a copy of the GNU General Public License along with
this program.  If not, see <http://www.gnu.org/licenses/>.
"""

from moinformat.input.common import Input
import codecs
import sys

class StandaloneInput(Input):

    "A stand-alone output context."

    name = "standalone"

    def read(self, stream=None):

        """
        Read from 'stream', or standard input if not specified, returning a
        character string.
        """

        stream = stream or sys.stdin
        f = codecs.getreader(self.encoding)(stream)
        return f.read()

    # Convenience methods.

    def get_attachment_filename(self, pagename, filename):

        """
        Prevent independent output by returning a filename of None corresponding
        to the given 'pagename' and any specified 'filename'.
        """

        return None

input = StandaloneInput

# vim: tabstop=4 expandtab shiftwidth=4
