#!/usr/bin/env python

"""
Graphviz region metadata parser. This only identifies metadata, with the actual
graph data being interpreted by Graphviz itself.

Copyright (C) 2018 Paul Boddie <paul@boddie.org.uk>

This program is free software; you can redistribute it and/or modify it under
the terms of the GNU General Public License as published by the Free Software
Foundation; either version 3 of the License, or (at your option) any later
version.

This program is distributed in the hope that it will be useful, but WITHOUT
ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
details.

You should have received a copy of the GNU General Public License along with
this program.  If not, see <http://www.gnu.org/licenses/>.
"""

from moinformat.parsers.common import ParserBase, choice, get_patterns, group, \
                                      optional
from moinformat.parsers.moin import MoinParser
from moinformat.tree.graphviz import Directive
from moinformat.tree.moin import Text

join = "".join

# Parser functionality.

class GraphvizParser(ParserBase):

    "A parser for Graphviz content, identifying format directives."

    format = "graphviz"

    # Parser handler methods.

    def parse_directive(self, region):

        "Handle format directives."

        directive = self.match_group("directive")
        key = self.match_group("key")
        value = self.match_group("value")

        self.add_node(region, Directive(key, value, directive))
        self.new_block(region)



    # Regular expressions.

    syntax = {
        # At start of line:

        "directive"     : choice((join((r"^#",                                      # #
                                        group("directive", r".*?$"),                # rest of line
                                        optional(group("extra", r"\n")))),          # nl (optional)

                                  # Legacy GraphvizParser directive syntax:

                                  join(("^//",                                      # //
                                        group("key", ".*?"),                        # text-excl-eq-nl
                                        optional(join(("=",                         # eq (optional)
                                                       group("value", ".*?")))),    # text-excl-nl (optional)
                                        "\n")))),                                   # nl

        "regionend"     : MoinParser.syntax["regionend"],
        }

    patterns = get_patterns(syntax)



    # Pattern details.

    region_pattern_names = ["directive", "regionend"]



    # Pattern handlers.

    parse_region_end = ParserBase.parse_region_end

    handlers = {
        "directive" : parse_directive,
        "regionend" : parse_region_end,
        }

parser = GraphvizParser

# vim: tabstop=4 expandtab shiftwidth=4
