<?xml version="1.0" ?>

<!--
Fix links, adding the index document filename if appropriate.

Copyright (c) 2019 Paul Boddie <paul@boddie.org.uk>

This program is free software; you can redistribute it and/or modify it under
the terms of the GNU General Public License as published by the Free Software
Foundation; either version 3 of the License, or (at your option) any later
version.

This program is distributed in the hope that it will be useful, but WITHOUT
ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
details.

You should have received a copy of the GNU General Public License along with
this program.  If not, see <http://www.gnu.org/licenses/>.
-->

<xsl:stylesheet xmlns:xsl="http://www.w3.org/1999/XSL/Transform" version="1.0" 
    xmlns:svg="http://www.w3.org/2000/svg"
    xmlns:xlink="http://www.w3.org/1999/xlink">
<xsl:output method="xml" indent="yes"
    doctype-public="-//W3C//DTD SVG 1.0//EN"
    doctype-system="http://www.w3.org/TR/2001/REC-SVG-20010904/DTD/svg10.dtd"/>

<!-- Accept the document index as a stylesheet parameter. -->

<xsl:param name="document_index"/>

<!-- Process link attributes only. -->

<xsl:template match="@xlink:href">
  <xsl:choose>

    <!-- Append the document index to the current value. -->

    <xsl:when test="$document_index">
      <xsl:attribute name="xlink:href">

        <!-- Generate the target resource. -->

        <xsl:choose>
          <xsl:when test="contains(., '#')">
            <xsl:value-of select="substring-before(., '#')"/>
          </xsl:when>
          <xsl:otherwise>
            <xsl:value-of select="."/>
          </xsl:otherwise>
        </xsl:choose>

        <!-- Add a path separator and document index. -->

        <xsl:if test="not(contains(., '#')) or substring-before(., '#')">
          <xsl:text>/</xsl:text>
          <xsl:value-of select="$document_index"/>
        </xsl:if>

        <!-- Add any fragment identifier. -->

        <xsl:if test="contains(., '#')">
          <xsl:text>#</xsl:text>
          <xsl:value-of select="substring-after(., '#')"/>
        </xsl:if>
      </xsl:attribute>
    </xsl:when>

    <!-- Preserve links without any document index specified. -->

    <xsl:otherwise>
      <xsl:copy/>
    </xsl:otherwise>
  </xsl:choose>
</xsl:template>

<!-- Copy all other nodes. -->

<xsl:template match="@*|node()">
  <xsl:copy>
    <xsl:apply-templates select="@*|node()"/>
  </xsl:copy> 
</xsl:template>

</xsl:stylesheet>
