#!/usr/bin/env python

"""
Graphviz document tree nodes.

Copyright (C) 2018 Paul Boddie <paul@boddie.org.uk>

This program is free software; you can redistribute it and/or modify it under
the terms of the GNU General Public License as published by the Free Software
Foundation; either version 3 of the License, or (at your option) any later
version.

This program is distributed in the hope that it will be useful, but WITHOUT
ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
details.

You should have received a copy of the GNU General Public License along with
this program.  If not, see <http://www.gnu.org/licenses/>.
"""

from moinformat.tree.moin import Node

class Directive(Node):

    "Format directive for Graphviz output."

    def __init__(self, key=None, value=None, directive=None):
        self.directive = directive

        if key or value:
            self.key = key
            self.value = value
        else:
            t = directive.split(None, 1)
            self.key = t[0]
            self.value = len(t) > 1 and t[1] or None

    def __repr__(self):
        return "Directive(%r, %r, %r)" % (self.key, self.value, self.directive)

    def prettyprint(self, indent=""):
        return "%sDirective: key=%r value=%r directive=%r" % (indent, self.key, self.value, self.directive)

    def to_string(self, out):
        out.directive(self.key, self.value, self.directive)

# vim: tabstop=4 expandtab shiftwidth=4
