#!/usr/bin/env python

"""
Copying utilities.

Copyright (C) 2019 Paul Boddie <paul@boddie.org.uk>

This program is free software; you can redistribute it and/or modify it under
the terms of the GNU General Public License as published by the Free Software
Foundation; either version 3 of the License, or (at your option) any later
version.

This program is distributed in the hope that it will be useful, but WITHOUT
ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
details.

You should have received a copy of the GNU General Public License along with
this program.  If not, see <http://www.gnu.org/licenses/>.
"""

from shutil import copy

def copy_attachments(parser, input, output, all=False):

    """
    For attachments referenced by the page processed by 'parser', copy them from
    'input' to 'output'. If 'all' is set to a true value, all stored attachments
    are copied; otherwise, only those explicitly linked in the page are copied.
    """

    pagename = parser.metadata.get("pagename")

    if not pagename:
        return

    # Obtain attachments.

    if not all:
        filenames = []
        for link_target in parser.link_targets:
            if link_target.get_type() == "attachment":
                filenames.append(link_target.get_identifier())
    else:
        input = parser.metadata.get_input()
        filenames = input.get_attachments(pagename)

    # Copy attachments.

    for filename in filenames:

        # Obtain the source location and the destination.

        input_filename = input.get_attachment_filename(pagename, filename)
        output_filename = output.get_attachment_filename(pagename, filename)

        # Copy the file if possible.

        if input_filename and output_filename:
            output.ensure_attachments(pagename)
            copy(input_filename, output_filename)

# vim: tabstop=4 expandtab shiftwidth=4
